--[[
	Author:  Azmaedus US-Arathor
	Version: 0.2 r10
	compatible with game version 6.0.3
	
	Change Log
	----------
	11/29/2014 - 0.1 	Creation of addon
	11/29/2014 - 0.1.1 	Support for tier 1, 2, and 3 garrison (Outpost, Fort, and Garrison)
	12/01/2014 - r8		1st commit to wowace.com (alpha)
	12/02/2014 - r9		Added multi-dimensional table to hold music filename references
	12/26/2014 - r9		Fixed issue with music not playing inside garrison buildings by changing the way to detect if we are inside Lunarfall
	12/27/2014 - r10	2 minute timer added between random playlist items.
						Dropdown music now functional per expansion selected.
						Fixed reload (musiclist) not loading in profile; forgot DB at the end of savedvars .toc
	
	Known Bugs
	----------
	11/29/2014 - r8		ReloadUI doesn't trigger event to play music.
	12/27/2014 - r10	[Fixed] - Reload UI doesn't load with selected playlist. Verified saved variable default profile in WFT.
]]

-- Load the Ace libraries --
GarrisonJukeBox = LibStub("AceAddon-3.0"):NewAddon("GarrisonJukeBox", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("GarrisonJukeBox")

-- globals
local gVersion = "0.2"
local gMusicPlaying = false
local gMusicIndex = 1
local gMaxMusic = 0
local LUNARFALL = 971

-- temp storage for select vars
local expac, music, jukebox = ""

-- Create the default settings for this addon --
local defaults = { 
    profile =  {
		musiclist = {},
		shuffle = true,
	},
}

--[[ temporary static music table; will be changed to a 2 dimensional table
local wowmusic = {
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AI_DraeneiWalkUni05.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AI_DraeneiWalkUni06.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AI_DraeneiWalkUni07r.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AI_DraeneiWalkUni08r.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AI_ExodarIntro01.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AI_ExodarWalkUni01.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AI_ExodarWalkUni02.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AI_ExodarWalkUni03.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AI_NagaWalkUni01.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AI_NagaWalkUni02.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AI_NagaWalkUni03.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AI_NagaWalkUni04.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AI_NagaWalkUni05.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AI_OwlkinWalkUni01.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AI_OwlkinWalkUni02.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AV_DraeneiWalkUni01.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AV_DraeneiWalkUni02.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AV_DraeneiWalkUni02r.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AV_DraeneiWalkUni03.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AV_DraeneiWalkUni03r.mp3",
	"Sound\\Music\\ZoneMusic\\Azuremyst\\AV_DraeneiWalkUni04.mp3",
	"Sound\\Music\\ZoneMusic\\Bloodmyst\\BI_SatyrWalkUni01r.mp3",
	"Sound\\Music\\ZoneMusic\\Bloodmyst\\BI_SatyrWalkUni02.mp3",
	"Sound\\Music\\ZoneMusic\\Bloodmyst\\BI_SatyrWalkUni03.mp3",
	"Sound\\Music\\ZoneMusic\\Bloodmyst\\BI_SatyrWalkUni04.mp3",
	"Sound\\Music\\ZoneMusic\\Bloodmyst\\BI_SatyrWalkUni05.mp3",
}
]]
-- multidimensional table holding music names
local musictable = {
	i1wow = {
		[1] = "Main Theme",
		"Darnassus",
		"Druid Grove",
		"Warrior Terrace",
		"IronForge",
		"Gnomeragon 1",
		"Gnomeragon 2",
		"Orgrimmar 1",
		"Orgrimmar 2",
		"Stormwind 1",
		"Stormwind 2",
		"Stormwind 3",
		"Stormwind 4",
		"Stormwind 5",
	},
	i2bc = {
		[100] = "Azuremist",
	},
	i3wotlk = {
		"",
	},
	i4cata = {
		"",
	},
	i5mop = {
		"",
	},
	i6wod = {
		"",
	},
}

-- table of desc/musicfile ref
local musicfiles = {
	-- classic
	[1] = "Sound\\Music\\GlueScreenMusic\\wow_main_theme.mp3",
	"Sound\\Music\\CityMusic\\Darnassus\\Darnassus Intro.mp3",
	"Sound\\Music\\CityMusic\\Darnassus\\Druid Grove.mp3",
	"Sound\\Music\\CityMusic\\Darnassus\\Warrior Terrace.mp3",
	"Sound\\Music\\CityMusic\\Ironforge\\IronForge Intro.mp3",
	"Sound\\Music\\CityMusic\\Gnomeragon\\gnomeragon01-zone.mp3",
	"Sound\\Music\\CityMusic\\Gnomeragon\\gnomeragon02-zone.mp3",
	"Sound\\Music\\CityMusic\\Orgrimmar\\orgrimmar01-zone.mp3",
	"Sound\\Music\\CityMusic\\Orgrimmar\\orgrimmar02-zone.mp3",
	"Sound\\Music\\CityMusic\\Stormwind\\stormwind04-zone.mp3",
	"Sound\\Music\\CityMusic\\Stormwind\\stormwind05-zone.mp3",
	"Sound\\Music\\CityMusic\\Stormwind\\stormwind06-zone.mp3",
	"Sound\\Music\\CityMusic\\Stormwind\\stormwind07-zone.mp3",
	"Sound\\Music\\CityMusic\\Stormwind\\stormwind08-zone.mp3",

	-- bc
	[100] = "Sound\\Music\\ZoneMusic\\Azuremyst\\AI_DraeneiWalkUni05.mp3",
	-- wotlk
	
	-- cata
	
	-- mop
	
	-- wod
}

-- Create the options variable to use in the config screen --
local options = {
	name = L["Garrison JukeBox Options"],
	desc = L["Build your playlist here."],
	handler = GarrisonJukeBox,
	type = "group",
	args = {
		expac = {
			type = "select",
			name = L["Expansion"],
			desc = L["Select the expansion."],
			style = "dropdown",
			values = "GetExpanssions",
			set = "SetExpac",
			get = "GetExpac",
			width = "normal",
			order = 1,
		},
		music = {
			type = "select",
			name = L["Music"],
			desc = L["Select the music."],
			style = "dropdown",
			values = "GetCurExpacMusicList",
			set = "SetMusic",
			get = "GetMusic",
			width = "normal",
			order = 2,
		},
		musicadd = {
			type = "execute",
			name = L["Add"],
			desc = L["Adds the music to your JukeBox list."],
			func = "AddMusic",
			width = "normal",
			order = 3,
		},
		musiclist = {
			type = "select",
			name = L["Your JukeBox"],
			desc = L["Your custom music list."],
			style = "dropdown",
			values = "GetJukeBoxList",
			set = "SetMusicProfile",
			get = "GetMusicProfile",
			width = "double",
			order = 50,
		},
		musicdelete = {
			type = "execute",
			name = L["Remove"],
			desc = L["Removes the selected music from your JukeBox list."],
			func = "DelMusic",
			width = "normal",
			order = 80,
		},
		musicdeleteall = {
			type = "execute",
			name = L["Remove All"],
			desc = L["Removes all music from your JukeBox list."],
			func = "DelMusicAll",
			width = "normal",
			order = 81,
		},
		shuffle = {
			type = "toggle",
			name = L["Shuffle"],
			desc = L["Randomly plays the music from your playlist."],

			set = function(info, value)
				GarrisonJukeBox.db.profile.shuffle = value
			end,

			get = function(info)
				return GarrisonJukeBox.db.profile.shuffle
			end,

			width = "full",
			order = 100,
		},
	},
}

function GarrisonJukeBox:GetExpanssions()
	return {
		i1wow = L["WoW Classic"],
		i2bc = L["The Burning Crusade"],
		i3wotlk = L["Wrath of the Lich King"],
		i4cata = L["Cataclysm"],
		i5mop = L["Mists of Pandaria"],
		i6wod = L["Warlords of Dreanor"],
	}
end

-- Expac get/set functions
function GarrisonJukeBox:SetExpac(info, value)
	GarrisonJukeBox.expac = value
end

function GarrisonJukeBox:GetExpac(info)
	return GarrisonJukeBox.expac
end

-- Music functions
function GarrisonJukeBox:SetMusic(info, value)
	GarrisonJukeBox.music = value
end

function GarrisonJukeBox:GetMusic(info)
	return GarrisonJukeBox.music
end

function GarrisonJukeBox:GetCurExpacMusicList()
	return musictable[self:GetExpac()]
end

-- Table Invert for query of key
function table_invert(t)
   local s={}
   for k,v in pairs(t) do
     s[v]=k
   end
   return s
end

-- Add, Remove, Remove All music functions
function GarrisonJukeBox:AddMusic()
	local xps = self:GetExpanssions()
	local xp = xps[self:GetExpac()]
	local mus = self:GetCurExpacMusicList()
	local mu = mus[self:GetMusic()]
	
	-- TODO: Will we allow duplicates?
	table.insert(GarrisonJukeBox.db.profile.musiclist, xp .. "/" .. mu)
	GarrisonJukeBox.jukebox = xp .. "/" .. mu
end

function GarrisonJukeBox:DelMusic()
	GarrisonJukeBox.db.profile.musiclist[GarrisonJukeBox.jukebox] = nil
end

function GarrisonJukeBox:DelMusicAll()
	GarrisonJukeBox.db.profile.musiclist = {}
	GarrisonJukeBox.jukebox = ""
end

-- Profile Music List
function GarrisonJukeBox:GetJukeBoxList()
	return GarrisonJukeBox.db.profile.musiclist
end

function GarrisonJukeBox:SetMusicProfile(info, value)
	GarrisonJukeBox.jukebox = value
end

function GarrisonJukeBox:GetMusicProfile(info)
	return GarrisonJukeBox.jukebox
end

-- OnLoad
function GarrisonJukeBox:OnLoad(frame)
	GarrisonJukeBox:RegisterEvent("ZONE_CHANGED_NEW_AREA", "OnEvent")
	GarrisonJukeBox:RegisterEvent("PLAYER_ENTERING_WORLD", "OnEvent")
	GarrisonJukeBox:RegisterEvent("PLAYER_LOGIN", "OnEvent")
	GarrisonJukeBox:RegisterEvent("ZONE_CHANGED_INDOORS", "OnEvent")
	GarrisonJukeBox:RegisterEvent("ZONE_CHANGED", "OnEvent")
	
	self:Print(L["Garrison JukeBox"] .. " " .. gVersion .. " " .. L["loaded!"])
	gMusicIndex = 1
	self.musicTimer = nil
	
end  

function GarrisonJukeBox:OnInitialize()
	GarrisonJukeBox.db = LibStub("AceDB-3.0"):New("GarrisonJukeBoxDB", defaults, true)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("GarrisonJukeBoxDB", options)
	GarrisonJukeBox.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("GarrisonJukeBoxDB", "Garrison JukeBox")

	self:RegisterChatCommand("gjb", "ChatCommand")
	self:RegisterChatCommand("garrisonjukebox", "ChatCommand")
	  
	self:Print(L["Garrison JukeBox"] .. " " .. gVersion .. " " .. L["initialized!"])
end

function GarrisonJukeBox:OnEvent(event, ...)
	if event == "ZONE_CHANGED_NEW_AREA" or event == "PLAYER_ENTERING_WORLD" or event == "PLAYER_LOGIN" or event == "ZONE_CHANGED_INDOORS" or event == "ZONE_CHANGED" then
		if GetCurrentMapAreaID() == LUNARFALL then
			if gMusicPlaying == false then
				gMaxMusic = table.getn(GarrisonJukeBox.db.profile.musiclist)
				if gMaxMusic > 0 then
					self:PlayMusicList()
				end
			end
		else 
			if gMusicPlaying == true then
				StopMusic()
				gMusicPlaying = false
				self:CancelTimer(self.musicTimer)
			end
		end
	end
end

function GarrisonJukeBox:TimerEvent()
	self:PlayMusicList()
end

-- break appart the musiclist item to find out what file to play in the musicfile table
function MusicItemIDFromList(s)
	local idx = 0
	local idm = 0
	
	local b = strfind(s, "/")
	local ns = strsub(s, 1, b-1)
	local xp = GarrisonJukeBox:GetExpanssions()
	local xpi = table_invert(xp)
	idx = xpi[ns]

	local mt = musictable[idx]
	local mti = table_invert(mt)
	ns = strsub(s, b+1)
	idm = mti[ns]
	
	return idm
end

function GarrisonJukeBox:PlayMusicList()
	gMaxMusic = table.getn(GarrisonJukeBox.db.profile.musiclist)
	if gMaxMusic > 0 then
		local id = math.random(1, gMaxMusic)
		local mli = GarrisonJukeBox.db.profile.musiclist[id]
		local mid = MusicItemIDFromList(mli)
		StopMusic()
		PlayMusic(musicfiles[mid])
		self:Print("Playing: " .. GarrisonJukeBox.db.profile.musiclist[id])
		gMusicPlaying = true
		self:CancelTimer(self.musicTimer)
		self.musicTimer = self:ScheduleTimer("TimerEvent", 120) -- fired after 2 minutes too go to the next music file
	end
end

function GarrisonJukeBox:ChatCommand(input)
  -- Open dialog if no parameters, else do something with the parameters
  if not input or input:trim() == "" then
    InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
  else
    LibStub("AceConfigCmd-3.0").HandleCommand(SpamageMeters, "gjb", "GarrisonJukeBox", input)
  end
end